////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// NDK 2.0 - Network Development Kit                                          //
//                                                                            //
// Authors: Sebastien Lachance                                                //
//                                                                            //
// E-mail:  netblitz@rocler.qc.ca                                             //
//                                                                            //
// -------------------------------------------------------------------------- //
//                                                                            //
// Permission to use, copy, modify, and distribute this software for any      //
// purpose and without fee is hereby granted. This is no guarantee about the  //
// use of this software. For any comments, bugs or thanks, please email us.   //
//                                                                            //
// -------------------------------------------------------------------------- //
//                                                                            //
// Targeted Platform: Any Windows version                                     //
//                                                                            //
// Compiled with:     Visual C++ 6.0                                          //
//                                                                            //
// Last modification: January 2002                                            //
//                                                                            //
// History:                                                                   //
//                                                                            //
// 1- First release of this file.                                             //
// 2- The class is renamed and some optimizations are applied. Hungarian      //
//    notation is used.                                                       //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Includes                                                                   //
////////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NDKUser.h"
#include "NDKServerSocket.h"


////////////////////////////////////////////////////////////////////////////////
// Constructors / Destructor                                                  //
////////////////////////////////////////////////////////////////////////////////

// Constructor.
CNDKUser::CNDKUser()
{
	m_lId = 0;
	m_pServerSocket = NULL;
	//IQ_egal_0 modifications START
	m_bCanSend = TRUE;
	m_dwExData = 0;
	m_bWaitForReply = FALSE;
	//IQ_egal_0 modifications END
}

//IQ_egal_0 modifications START

// Constructor with initialization.
CNDKUser::CNDKUser(long lId, CNDKServerSocket* pServerSocket)
{
	ASSERT((lId > 0) && (pServerSocket != NULL));

	m_lId = lId;
	m_pServerSocket = pServerSocket;
	//IQ_egal_0 modifications START
	m_dwExData = 0;
	m_bCanSend = TRUE;
	m_bWaitForReply = FALSE;
	//IQ_egal_0 modifications END
}

//IQ_egal_0 modifications END

// Destructor.
CNDKUser::~CNDKUser()
{
}


////////////////////////////////////////////////////////////////////////////////
// Attributes                                                                 //
////////////////////////////////////////////////////////////////////////////////

// Gets the Id.
long CNDKUser::GetId() const
{
	return m_lId;
}


// Returns if the socket is the same as specified.
BOOL CNDKUser::IsSocketEqual(CNDKServerSocket* pServerSocket) const
{
	return m_pServerSocket == pServerSocket;
}


// Returns if the buffer of the socket is empty.
BOOL CNDKUser::IsSocketBufferEmpty() const
{
	ASSERT(m_pServerSocket != NULL);

	BOOL bResult = TRUE;

	if (m_pServerSocket != NULL)
		bResult = m_pServerSocket->IsBufferEmpty();

	return bResult;
}

//IQ_egal_0 modifications START

//Returns the extended data
DWORD CNDKUser::GetExData()
{
	return m_dwExData;
}

//Sets the extended data, returning the old one
DWORD CNDKUser::SetExData(DWORD dwExData)
{
	DWORD dwOldExData = m_dwExData;
	m_dwExData = dwExData;
	return dwOldExData;
}

//Returns the send state
BOOL CNDKUser::GetSendState()
{
	return m_bCanSend;
}

//Sets the send state, returning the old one
BOOL CNDKUser::SetSendState(BOOL bCanSend)
{
	BOOL bOldCanSend = m_bCanSend;
	m_bCanSend = bCanSend;
	return bOldCanSend;
}
	
//Returns the wait for reply state
BOOL CNDKUser::GetWaitForReplyState()
{
	return m_bWaitForReply;
}

//Sets the wait for reply state, returning the old one
BOOL CNDKUser::SetWaitForReplyState(BOOL bWaitForReply)
{
	BOOL bOldWaitForReply = m_bWaitForReply;
	m_bWaitForReply = bWaitForReply;
	return bOldWaitForReply;
}

//IQ_egal_0 modifications END


////////////////////////////////////////////////////////////////////////////////
// Operations                                                                 //
////////////////////////////////////////////////////////////////////////////////

// Sends a message.
BOOL CNDKUser::SendMessage(CNDKMessage& message)
{
	ASSERT(m_pServerSocket != NULL);

	//IQ_egal_0 modifications START
	if (!m_bCanSend)
		return TRUE;
	//IQ_egal_0 modifications END

	BOOL bResult = FALSE;

	if (m_pServerSocket != NULL)
	{
		TRY
		{
			m_pServerSocket->SendMessage(message);
			bResult = TRUE;
		}
		CATCH(CFileException, e)
		{
			m_pServerSocket->Close();
		}
		CATCH(CArchiveException, e)
		{
			m_pServerSocket->Close();
		}
		END_CATCH
	}

	return bResult;
}


// Reads a message.
BOOL CNDKUser::ReadMessage(CNDKMessage& message)
{
	ASSERT(m_pServerSocket != NULL);

	BOOL bResult = FALSE;

	if (m_pServerSocket != NULL)
	{
		TRY
		{
			m_pServerSocket->ReceiveMessage(message);
			bResult = TRUE;
		}
		CATCH(CFileException, e)
		{
			m_pServerSocket->Close();
		}
		END_CATCH
	}

	return bResult;
}


// Closes the socket.
void CNDKUser::CloseSocket()
{
	if (m_pServerSocket != NULL)
	{
		m_pServerSocket->ShutDown();
		m_pServerSocket->Close();

		delete m_pServerSocket;
		m_pServerSocket = NULL;
	}
}


////////////////////////////////////////////////////////////////////////////////
// Operators                                                                  //
////////////////////////////////////////////////////////////////////////////////

// Comparison operator.
BOOL CNDKUser::operator==(const CNDKUser& user) const
{
	return (m_lId == user.m_lId) && (m_pServerSocket == user.m_pServerSocket);
}
